<?php

namespace Litepie\Team\Http\Controllers;

use Exception;
use Litepie\Team\Actions\UserWorkflow;
use Litepie\Team\Http\Requests\UserWorkflowRequest;
use Litepie\Team\Http\Resources\UserResource;
use Litepie\Team\Models\User;
use Litepie\Http\Controllers\ActionController as BaseController;

/**
 * Workflow controller class.
 *
 */
class UserWorkflowController extends BaseController
{
    /**
     * Action controller function for user.
     *
     * @param Model $user
     * @param action next action for the user.
     *
     * @return Response
     */
    public function __invoke(UserWorkflowRequest $request, User $user, $trasition)
    {
        try {
            $request = $request->all();
            $user = UserWorkflow::run($trasition, $user, $request);
            $data = new UserResource($user);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('team::user.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('team/user/' . $user->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('team/user/' . $user->getRouteKey()))
                ->redirect();
        }
    }


    /**
     * Get the user and user for the workflow.
     *
     * @param PublicRequest $request
     * @param string $user
     * @param string $user
     *
     * @return Response
     */
    public function get(PublicRequest $request, $user, $user)
    {
        if (!$request->hasValidSignature()) {
            abort(403, 'This url is expired!');
        }

        $auth = Auth::guard()->getProvider()->getModel();
        $user = $auth::findBySignedId($user);
        Auth::guard()->login($user);
        $user = User::findBySignedId($user);
        $transitions = $this->transitions($user, $user);
        $user = user();
        Auth::logout();
        return $this->response->setMetaTitle(trans('team::user.name') . ' ' . trans('Approvals'))
            ->view('team::public.user.workflow')
            ->layout('mobile')
            ->data(compact('user', 'user', 'transitions'))
            ->output();
    }

    /**
     * Display the workflow form for the user.
     *
     * @param PublicRequest $request
     * @param string $user
     * @param string $user
     *
     * @return Response
     */
    public function post(PublicRequest $request, $user, $user)
    {
        if (!$request->hasValidSignature()) {
            abort(403, 'This url is expired!');
        }
        $transition = $request->transition;
        $auth = Auth::guard()->getProvider()->getModel();
        $user = $auth::findBySignedId($user);
        Auth::guard()->login($user);
        $user = User::findBySignedId($user);
        $user = user();
        $user = UserWorkflow::run($transition, $user, $request->all());
        Auth::logout();

        return response()->json(
            [
                'status' => 'success',
                'url' => $request->url(),
            ]
        );
    }

    private function transitions($user, $user)
    {
        $transitions = [];
        foreach ($user->workflow()->transitions($user) as $key => $value) {
            $name = $value->getName();
            $array['url'] = URL::temporarySignedRoute('litecms.user.workflow', now()->addMinutes(3), [
                'guard' => 'admin',
                'transition' => $name,
                'user' => $user->getSignedId(now()->addMinutes(3)),
                'user' => $user->getSignedId(now()->addMinutes(3)),
                'trans' => 'en',
            ]
            );
            $array['name'] = $name;
            $array['key'] = $name;
            $array['form'] = $user->workflow()->form($value);
            $array['label'] = trans('team::user.workflow.' . $name);
            $transitions[] = $array;
        }
        return $transitions;
    }
}
